﻿

module("VOICE")

ASSERT(type(voice_debug_log) == "function", "Missing voice_debug_log function.")
ASSERT(type(voice_debug_log_table) == "function", "Missing voice_debug_log_table function.")

ASSERT(type(wjoin) == "function", "Missing wjoin function.")
ASSERT(type(wsplit) == "function", "Missing wsplit function.")
ASSERT(type(getFirstNChar) == "function", "Missing getFirstNChar function.")
ASSERT(type(getLastNChar) == "function", "Missing getLastNChar function.")

ASSERT(type(transform_and_format) == "function", "Missing transform_and_format function.")
ASSERT(type(transform_chain) == "function", "Missing transform_chain function.")
ASSERT(type(table_concat) == "function", "Missing table_concat function.")
ASSERT(type(transform_pattern_match) == "function", "Missing transform_pattern_match function.")

ASSERT(type(transform.replace_sentence) == "table", "Missing transform.replace_sentence table.")
ASSERT(type(transform.roadname_abbrev_table) == "table", "Missing transform.roadname_abbrev_table table.")
ASSERT(type(transform.direction_abbrev) == "table", "Missing transform.direction_abbrev table.")


local function format_number(str)
	if type(str) ~= "wstring" then
		ASSERT(FALSE, "The input of format_number must be wstring. Type: "..type(str))
		return str
	end
	local formatter = transform.number_formatters[#str]
	if formatter then
		local rep
		for _, v in ipairs(formatter) do
			str, rep = wstring.gsub(str, v[1], v[2])
			if rep > 0 then
				break
			end
		end
		for _, v in ipairs(transform.oh_formatters) do
		rep = 1
			while rep > 0 do
				str, rep = wstring.gsub(str, v[1], v[2])
			end
		end
	end
	return str
end

local function is_road_number(str)
	return transform_pattern_match(str, transform.roadnumber_typ_patterns)
end



local function format_road_number(roadnum)
	local res
	if is_road_number(roadnum) then
		roads = transform_roadnumber_explode_eu(roadnum)
		for k, str in ipairs(roads) do
			
			str = wstring.upper(str)
			
			str = wstring.gsub(str, L"(%a)-(%a)", L"%1 %2")
			_,_,head,num,tail = wstring.find(str, L"^(.-)(%d+)(.-)$")
			
			if head == L"A" then
				head = L"Ay"
			end
			
			num = format_number(num)
			
			tail = transform.direction_abbrev:transform(tail)
			
			str = table_concat({head,num,tail}, L" ")
			
			str = wstring.gsub(str, L"%s%s+", L" ")
			
			roads[k] = str
		end
		res = table_concat(roads, L"; ")
	else
		res = transform.roadname_abbrev_table:transform(roadnum)
	end
	return res
end

local function format_road_name(str)
	if is_road_number(str) then
		
		str = format_road_number(str)
	else
		
		str = transform.roadname_abbrev_table:transform(str)
	end
	return str
end


local function format_signpost_exitnumber(str)
	local _, _, prefix, num, postfix = wstring.find(str, L"^(.-)(%d+)(.-)$")
	if num then
		num = format_number(num)
	else
		voice_debug_log(L"No number in format_signpost_exitnumber: " .. str, 2)
	end
	
	if prefix then
		prefix = wstring.gsub(wstring.upper(prefix), L"EXIT%s*", L"")
	end
	
	if postfix then
		postfix = wstring.gsub(postfix, L"[/-]+", L", ")
	end
	
	return table_concat({prefix, num, postfix}, L" ")
end

local function signpost_exitnumber(data, idx)
	return transform_and_format(data[idx].signpost.exitnumber, format_signpost_exitnumber, transform.destname.EXITNUMBER)
end

local function signpost_exitname(data, idx)
	return transform_and_format(data[idx].signpost.exitname, nil, transform.destname.EXITNAME)
end

local function signpost_destination(data, idx)
	return transform_and_format(data[idx].signpost.destination, transform.roadname_abbrev_table, transform.destname.DESTINATION)
end

local function signpost_settlement(data, idx)
	return transform_and_format(data[idx].signpost.settlement, transform.direction_abbrev, transform.destname.SETTLEMENT)
end

local function signpost_roadnumber(data, idx)
	if data[idx].signpost then
		local roadNumber = data[idx].signpost.roadnumber
		
		local signRoadBranch = data[idx].signpost.sign_road_branch
		if roadNumber and signRoadBranch then
			return transform_and_format(data[idx].signpost.roadnumber, format_road_number, transform.destname.ROADNAME)
		elseif roadNumber and not signRoadBranch then
			if data[idx].road then
				if data[idx].road.number and data[idx].road.number.text ~= roadNumber.text then
					return transform_and_format(data[idx].road.number, format_road_number, transform.destname.ROADNAME) .. L", " .. transform_and_format(roadNumber, format_road_number, transform.destname.SETTLEMENT)
				else
					return transform_and_format(data[idx].road.number, format_road_number, transform.destname.ROADNAME)
				end
			else
				return transform_and_format(roadNumber, format_road_number, transform.destname.SETTLEMENT)
			end
		else
			if data[idx].road and data[idx].road.number then
				transform_and_format(data[idx].road.number, format_road_number, transform.destname.ROADNAME)
			end
		end
	end
end

local function road_name(data, idx)
	return transform_and_format(data[idx].road.name, format_road_name)
end

local function road_number(data, idx)
	return transform_and_format(data[idx].road.number, format_road_number)
end

function format_destname(data, idx)
	local t = {}
	local out = L""
	if data[idx].signpost then
		t = transform_chain(t, signpost_exitname,		data, idx)
		t = transform_chain(t, signpost_exitnumber,		data, idx)
		t = transform_chain(t, signpost_roadnumber,		data, idx)
		t = transform_chain(t, signpost_destination,	data, idx)
		t = transform_chain(t, signpost_settlement,		data, idx)
		out = table_concat(t, L", ")
	elseif data[idx].road then
		t = transform_chain(t, road_number,	data, idx)
		t = transform_chain(t, road_name,	data, idx)
		out = wstring.format(transform.destname.ROADNAME, table_concat(t, L", "))
	end
	return out
end

function format_sentence(str)
	for key, value in ipairs(transform.replace_sentence) do
		str = wstring.gsub(str, value[1], value[2])
	end
	return str
end

function format_streetname(data, idx)
	local t = {}
	if data[idx].road then
		t = transform_chain(t, road_number,	data, idx)
		t = transform_chain(t, road_name,	data, idx)
	end
	return table_concat(t, L", ")
end

function route_summary_format_road_name(data)
	return transform_and_format(data, format_road_number)
end

function route_summary_format_street_name(data)
	return transform_and_format(data, transform.roadname_abbrev_table)
end

function route_summary_format_bridge_tunnel(data)
	return transform_and_format(data, transform.roadname_abbrev_table)
end

function route_summary_format_order(data)
	return transform_and_format(data, transform.roadname_abbrev_table)
end


function traffic_event_supported()
	return true
end

function traffic_event(DescKey, data)
	local str = translate_voice(DescKey)
	ASSERT(MODEL.regional.is_it_voice_localizable(DescKey), "Missing TrafficEvent dictionary.voice key:" .. DescKey)
	
	if data.roadnumber then 
		local road = transform_and_format(data.roadnumber, format_road_number, transform.event.ON)
		if data.from and not data.to then 
			str = str..road..transform_and_format(data.from, nil, transform.event.NEAR)
		elseif data.from and data.to then 
			str = str..road..transform_and_format(data.from, nil, transform.event.BETWEEN)..transform_and_format(data.to, nil, transform.event.AND)
		else 
			str = str..road
		end
	
	elseif data.roadname then
		local road = transform_and_format(data.roadname, transform.roadname_abbrev_table, transform.event.ON)
		if data.from and not data.to then
			local _,_, from, to = wstring.find(data.from.text, L"^([^,]+),(.+)$") 
			if from and to then 
				str = str..road..wstring.format(transform.event.AT_INTERSECTION_OF, from)..wstring.format(transform.event.AND, to)
			else 
				str = str..road..transform_and_format(data.from, nil, transform.event.AT_INTERSECTION_OF)
			end
		elseif data.from and data.to then
			str =  str..road..transform_and_format(data.from, nil, transform.event.BETWEEN)..transform_and_format(data.to, nil, transform.event.AND)
		else
			str = str..road
		end
	
	else
		if data.from and not data.to then 
			str =  str..road..transform_and_format(data.from, nil, transform.event.NEAR)
		elseif data.from and data.to then 
			str =  str..road..transform_and_format(data.from, nil, transform.event.BETWEEN)..transform_and_format(data.to, nil, transform.event.AND)
		else
		
		end
	end
	str = str..L"."
	return str
end


